
##
## Rotations
##

def rotation_point(O,a,P):
	x = O.x + math.cos(a)*(P.x-O.x) - math.sin(a)*(P.y-O.y)
	y = O.y + math.sin(a)*(P.x-O.x) + math.cos(a)*(P.y-O.y)
	return Point(x,y)

def rotation_vecteur(O,a,u):
	return Vecteur(rotation_point(O,a,u.P),rotation_point(O,a,u.Q))

def rotation_segment(O,a,S):
	return Segment(rotation_point(O,a,S.P),rotation_point(O,a,S.Q))

def rotation_droite(O,a,D):
	return Droite(rotation(O,a,D.point()),
		      rotation(O,a,D.vecteur_directeur()))

def rotation_angle(O,a,A):
	return Angle(rotation_point(O,a,A.A),
		     rotation_point(O,a,A.O),
		     rotation_point(O,a,A.B))

def rotation_cercle(O,a,C):
	return Cercle(rotation_point(O,a,C.centre),C.rayon)

def rotation_spline(O,a,S):
	return Spline(map(lambda p, O=O, a=a: rotation(O,a,p),
			  S.points))

# Reste encore conique, polygone, demi-droite

def rotation(O,a,P):
	if CLASS(P,Point):
		return rotation_point(O,a,P)
	elif CLASS(P,Vecteur):
		return rotation_vecteur(O,a,P)
	elif CLASS(P,Segment):
		return rotation_segment(O,a,P)
	elif CLASS(P,Droite):
		return rotation_droite(O,a,P)
	elif CLASS(P,Angle):
		return rotation_angle(O,a,P)
	elif CLASS(P,Cercle):
		return rotation_cercle(O,a,P)
	elif CLASS(P,Spline):
		return rotation_spline(O,a,P)
	else:
		raise JeNeSaisPas("effectuer de rotation de", P.__class__)

def symetrie_centrale(O,objet):
	return rotation(O,math.pi,objet)

##
## Homothtie
##

def homothetie_point(O,l,A):
	x = O.x + l*(A.x-O.x)
	y = O.y + l*(A.y-O.y)
	return Point(x,y)

def homothetie_vecteur(O, k, v):
	return Vecteur(homothetie_point(O,k,v.P),
		       homothetie_point(O,k,v.Q))

def homothetie_cercle(O,k,C):
	return Cercle(homothetie(O,k,C.centre),
		      k*C.rayon)

def homothetie_segment(O,k,S):
	return Segment(homothetie(O,k,S.P),homothetie(O,k,S.Q))

def homothetie_droite(O,k,D):
	return Droite(homothetie(O,k,D.point()),
		      homothetie(O,k,D.vecteur_directeur()))

def homothetie_angle(O,k,A):
	return Angle(homothetie_point(O,k,A.A),
		     homothetie_point(O,k,A.O),
		     homothetie_point(O,k,A.B))

def homothetie_spline(O,k,S):
	return Spline(map(lambda p, O=O, k=k: homothetie(O,k,p),
			  S.points))


def homothetie(centre, rapport, objet):
	if CLASS(objet, Point):
		return homothetie_point(centre, rapport, objet)
	elif CLASS(objet, Vecteur):
		return homothetie_vecteur(centre, rapport, objet)
	elif CLASS(objet, Cercle):
		return homothetie_cercle(centre, rapport, objet)
	elif CLASS(objet, Droite):
		return homothetie_droite(centre,rapport,objet)
	elif CLASS(objet, Segment):
		return homothetie_segment(centre,rapport,objet)
	elif CLASS(objet, Angle):
		return homothetie_angle(centre,rapport,objet)
	elif CLASS(objet, Spline):
		return homothetie_spline(centre,rapport,objet)
	else:
		raise JeNeSaisPas("calculer l'homothetie de", objet.__class__)


##
## Translation
##

def translation_point(u,A):
	return Point(A.x+u.x, A.y+u.y)

def translation_cercle(v,C):
	D = copy(C)
	D.centre = translation_point(v,C.centre)
	return D

def translation_droite(v,D):
	E = copy(D)
	E.c = D.c - (D.a*v.x+D.b*v.y)
	return E

def translation_segment(v,S):
	# PROBLME : Quand je cre un nouvel objet, 
	#            je ne garde pas les attribus de l'ancien...
	return Segment(translation(v,S.P), translation(v,S.Q))

def translation_vecteur(v,V):
	return Vecteur(translation(v,V.P), translation(v,V.Q))

def translation_angle(v,A):
	return Angle(translation(v,A.A), translation(v,A.O), translation(v,A.B))

def translation_spline(v,S):
	return Spline(map(lambda p, v=v: translation(v,p),
			  S.points))

def translation_demi_droite(v,D):
	E = copy(D)
	E.A = translation_point(v,D.A)
	return E

def translation(v,A):
	if not CLASS(v,Vecteur):
		raise Probleme("Tu t'es gourr dans l'ordre des arguments")
	elif CLASS(A,Point):
		return translation_point(v,A)
	elif CLASS(A, Cercle):
		return translation_cercle(v,A)
	elif CLASS(A, Droite):
		return translation_droite(v,A)
	elif CLASS(A, Vecteur):
		return translation_vecteur(v,A)
	elif CLASS(A, Segment):
		return translation_segment(v,A)
	elif CLASS(A, Angle):
		return translation_angle(v,A)
	elif CLASS(A, Spline):
		return translation_spline(v,A)
	elif CLASS(A, DemiDroite):
		return translation_demi_droite(v,A)
	else:
		raise JeNeSaisPas("translater de", A.__class__)

##
## Symtrie axiale
##

def symetrie_axiale_point(D,A):
	E = perpendiculaire(D,A)
	d = distance(A,D)
	C = Cercle(A, 2*d)
	(I,J) = intersection(C,E)
	if distance(I,D) < distance(J,D):
		return I
	else:
		return J

def symetrie_axiale_droite(D,A):
	return Droite(symetrie_axiale(D,A.point()),
		      symetrie_axiale(D,A.vecteur_directeur()))

def symetrie_axiale_segment(D,A):
	return Segment(symetrie_axiale_point(D,A.P), symetrie_axiale_point(D,A.Q))

def symetrie_axiale_vecteur(D,A):
	return Vecteur(symetrie_axiale_point(D,A.P), symetrie_axiale_point(D,A.Q))

def symetrie_axiale_angle(D,A):
	return Angle(symetrie_axiale_point(D,A.B), # Attention  l'ordre !
		     symetrie_axiale_point(D,A.O), # l'angle image est dans le sens _oppos_...
		     symetrie_axiale_point(D,A.A))

def symetrie_axiale_cercle(D,A):
	return Cercle(symetrie_axiale_point(D,A.centre), A.rayon)

def symetrie_axiale_spline(D,S):
	return Spline(map(lambda p, D=D: symetrie_axiale(D,p),
			  S.points))

def symetrie_axiale(D,objet):
	if not CLASS(D, Droite):
		raise Erreur("Tu t'es gourr dans l'ordre des arguments")
	elif CLASS(objet, Point):
		return symetrie_axiale_point(D,objet)
	elif CLASS(objet, Droite):
		return symetrie_axiale_droite(D,objet)
	elif CLASS(objet, Segment):
		return symetrie_axiale_segment(D,objet)
	elif CLASS(objet, Vecteur):
		return symetrie_axiale_vecteur(D,objet)
	elif CLASS(objet, Angle):
		return symetrie_axiale_angle(D,objet)
	elif CLASS(objet, Cercle):
		return symetrie_axiale_cercle(D,objet)
	elif CLASS(objet, Spline):
		return symetrie_axiale_spline(D,objet)
	else:
		raise("Je ne sais pas calculer la stmetrie de cette chose...")


##
## Inversion
##

def inversion_point(O,k,M):
	v = Vecteur (O,M)
	d = distance(O,M)
	v = homothetie(O, k/(d*d), v)
	N = translation(v,O)
	return N

def inversion_cercle(O,k,C):
	if zero(C.puissance(O)):
		return C.tangente(O)
	else:
#		C.trace("",0,"epais")
		(A,B) = intersection(C, Droite(O,C.centre))
#		A.trace("",0,"","rouge")
#		B.trace("",0,"","rouge")
		AA = inversion(O,k,A)
		BB = inversion(O,k,B)
#		AA.trace("",0,"","bleu")
#		BB.trace("",0,"","bleu")
#		Segment(AA,BB).trace()
		I = milieu(AA, BB)
		return Cercle(I, distance(I, AA))

def inversion_droite(O,k,D):
	if zero(distance(O,D)):
		return D
	else:
		return cercle_circonscrit(
			inversion(O,k,D.point()),
			inversion(O,k,D.point()),
			inversion(O,k,D.point())
			)

def inversion_spline(O,k,S):
	return Spline(map(lambda p, O=O, k=k: inversion(O,k,p),
			  S.points))


def inversion_de_centre_et_de_rapport(O,k,A):
	if CLASS(A,Point):
		return inversion_point(O,k,A)
	elif CLASS(A,Cercle):
		return inversion_cercle(O,k,A)
	elif CLASS(A,Droite):
		return inversion_droite(O,k,A)
	elif CLASS(A,Spline):
		return inversion_spline(O,k,A)
	else:
		raise JeNeSaisPas("inverser", A.__class__)

def inversion_par_rapport_a_un_cercle(C,A):
	return inversion_de_centre_et_de_rapport(C.centre, C.rayon*C.rayon, A)

def inversion(a,b,c=None):
	if c==None:
		if CLASS(a,Cercle):
			return inversion_par_rapport_a_un_cercle(a,b)
		else:
			raise JeNeSaisPas("inverser par rapport ", a.__class__)
	else:
		return inversion_de_centre_et_de_rapport(a,b,c)
		
